package com.nb.pi.processor.beans;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.extractor.WordExtractor;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

import com.nb.pi.db.beans.Account;
import com.nb.pi.db.beans.AccountId;
import com.nb.pi.db.beans.Disclosures;
import com.nb.pi.db.beans.DisplayType;
import com.nb.pi.db.beans.IndHdr;
import com.nb.pi.db.beans.Source;
import com.nb.pi.db.beans.Template;
import com.nb.pi.db.beans.TemplatePeriods;
import com.nb.pi.db.beans.TemplatePeriodsId;
import com.nb.pi.db.beans.TemplateProducts;
import com.nb.pi.db.beans.TemplateProductsId;
import com.nb.pi.db.beans.TemplateStats;
import com.nb.pi.db.beans.TemplateStatsId;
import com.nb.pi.manager.AccountLongNameConverter;
import com.nb.pi.manager.DisclosuresManager;
import com.nb.pi.manager.DisplayTypeConverter;
import com.nb.pi.manager.IndCtrlManager;
import com.nb.pi.manager.IndHdrMgrConverter;
import com.nb.pi.manager.ProductBenchConverter;
import com.nb.pi.manager.SourceConverter;
import com.nb.pi.manager.TemplateConverter;
import com.nb.pi.manager.TemplateManager;
import com.nb.pi.processor.CalendarPeriod;
import com.nb.pi.processor.PeriodType;
import com.nb.pi.processor.Processor;
import com.nb.pi.processor.statistics.Statistic;
import com.nb.pi.processor.statistics.StatisticsValues;
import com.nb.pi.util.SelectItemComparator;
import com.nb.pi.web.beans.UserSecurityBean;

public class QueryToolBean extends ProcessorInputBean {
	private static Logger logger = Logger.getLogger(QueryToolBean.class);
	private List<SelectItem> shortNameProductBenchList;
	private List<SelectItem> productBenchList;
	private List<SelectItem> benchmarkList;
	private List<SelectItem> riskFreeList;
	private List<SelectItem> sourceList;
	private Source source;
	private List<SelectItem> displayTypeList;
	private DisplayType displayType;
	private List<SelectItem> periodList = new ArrayList<SelectItem>();
	private List<SelectItem> monthList;
	private List<TrailingPeriods> allTrailingPeriods;
	private List<Statistic> allStatistics;
	private List<Statistic> riskFreeStatistics;
	private List<SelectItem> sortPeriodList = new Vector<SelectItem>();
	private TrailingPeriods sortPeriod;
	private List<SelectItem> sortStatisticList = new Vector<SelectItem>();
	private Statistic sortStatistic;
	private List<SelectItem> templateList = new Vector<SelectItem>();
	private String templateName;
	private List<SelectItem> templateOwnersList = new Vector<SelectItem>();
	private String templateOwner;
	
	// maintain the last selected trailing periods 
	// - parent trailing periods is the one to use for calculation, this is for save state
	protected List<TrailingPeriods> selectedTrailingPeriods;
	private Vector<Boolean> prodCheckedList = new Vector<Boolean>();
	private Boolean prodCheckedAll = false;
	
	private boolean calendarVisible = false;
	private boolean customVisible = false;
	private boolean trailingVisible = true;
	private boolean disallowSectorSelection = false;
	private boolean disallowGrossSelection = false;
	private boolean disallowPeriodicitySelection = false;
	private boolean allowMktSelection = false;
	private boolean allowFundAndPlanTypeSelection = false;
	private boolean allowSorting = true;
	private boolean hasExcelData = false;
	private boolean hasNonGross = false;
	private boolean disallowGroupbyBenchmark = false;
	private boolean isOnlyBenchmarks = true;
	
	private ProductBenchRiskFree curProdBenchRF = new ProductBenchRiskFree();
	
	private TimePeriod curTimePeriod = new TimePeriod();

	private Vector<StatisticsOutputData> allRetData = new Vector<StatisticsOutputData>();

	private String asOfMonth;
	private String asOfYear;

	private int digitsDecimal = 2;
	
	protected String reportType;
	private AccountLongNameConverter accountManager;
	private ProductBenchConverter productBenchManager;
	private IndHdrMgrConverter benchmarkManager;
	private SourceConverter sourceConverter;
	private DisplayTypeConverter displayTypeConverter;
	private TemplateConverter templateConverter;
	private Processor statProc = new Processor();
	private TemplateManager templateMgr = new TemplateManager();
	private UserSecurityBean usrSecurityBean;
	
	private int lastProcessIndex = -1;
	final private int retRandNum;
	private String disclosure = "";
	
	
	public int getRetRandNum() {
		return retRandNum;
	}

	private HashMap<String, Integer> monthLookup;
	static private final String MONTHS[] = { "January", "February", "March", "April", "May", "June",
		"July", "August", "September", "October", "November", "December" };

	public QueryToolBean() {
		retRandNum = (int)Math.floor((Math.random() * 10000));
		curTimePeriod.periodType = PeriodType.TRAILING;
		statistics = new Vector<Statistic>();
		allStatistics = Statistic.getVisibleStatistics();
		selectedTrailingPeriods = new Vector<TrailingPeriods>();
		final Vector<TrailingPeriods> tps = new Vector<TrailingPeriods>();
		tps.addAll(Arrays.asList(TrailingPeriods.values()));
		TrailingPeriods tpYTD = tps.remove(3); //REMEMBER TO UPDATE
		allTrailingPeriods = tps;
		monthLookup = new HashMap<String, Integer>();
		monthList = new Vector<SelectItem>();
		int i = 0;
		for(final String month : MONTHS) {
			monthList.add(new SelectItem(month, month));
			monthLookup.put(month, i);
			i++;
		}
		GregorianCalendar cal = new GregorianCalendar();
		int beginYear = cal.get(Calendar.YEAR);
		for (i = beginYear; i > 1938; i-- ) {
			final Integer iObj = new Integer(i);
			periodList.add( new SelectItem(iObj.toString(), iObj.toString()));
		}
		curTimePeriod.periodicity = Periodicity.MONTHLY;
		grossnet = GrossNet.NET;
		sortorder = SortOrder.DESC; // Normally want to see top performance on top
		groupbyBenchmark = GroupbyBenchmark.YES;
		benchmarkRequired = BenchmarkRequired.YES;
		cal.add(Calendar.MONTH, -1);
		curTimePeriod.asOf = cal;
		// We will initialize custom time periods as well to prevent validation error
		GregorianCalendar curCal = (GregorianCalendar)cal.clone();
		curCal.set(Calendar.DAY_OF_MONTH, 1);
		curTimePeriod.custPeriod.setFrom(curCal.getTime());
		curTimePeriod.custPeriod.setTo(curCal.getTime());
		
		final int curMonth = curTimePeriod.asOf.get(Calendar.MONTH);
		asOfMonth = MONTHS[curMonth];
		asOfYear = new String(Integer.toString(curTimePeriod.asOf.get(Calendar.YEAR)));
		final int insIdx;
		if (curMonth + 1 <= 3) {
			insIdx = 1;
		} else if (curMonth + 1 <= 6){
			insIdx = 2;
		} else {
			insIdx = 3;
		}
		tps.add(insIdx, tpYTD);
		
		prodBenchRFs = new Vector<ProductBenchRiskFree>();
		timePeriods = new Vector<TimePeriod>();
		riskFreeStatistics = Statistic.getRiskFreeStatistics();
		prodCheckedAll = false;
	}
	
	public void periodTypeChange(ValueChangeEvent vce) {
		logger.debug("<changePeriodType>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();
		final PeriodType pt = (PeriodType)vce.getNewValue();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			if (PeriodType.TRAILING == pt) {
				trailingVisible = true;
				calendarVisible = false;
				customVisible = false;
				allowSorting = true;
			}
			if(PeriodType.CALENDAR == pt) {
				calendarVisible = true;
				trailingVisible = false;
				customVisible = false;
				allowSorting = false;
				//prevent any trailing periods from being processed
			}
			if(PeriodType.CUSTOM == pt) {
				calendarVisible = false;
				trailingVisible = false;
				customVisible = true;
				allowSorting = false;
				//prevent any trailing periods from being processed
			}
		}		
		
		logger.debug("</changePeriodType>");
	}
	
	public void sectorChange(ValueChangeEvent vce) {
		logger.debug("<changeSector>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();
		final SectorEnum sector = (SectorEnum)vce.getNewValue();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			if  ( SectorEnum.TotalPortfolio == sector)
			{
				if ( hasNonGross )
					grossnet = GrossNet.NET;
				else
					disallowGrossSelection = false;
					
			}
			else // Equity or Fixed Income sectors can be only reported as gross
			{
				grossnet = GrossNet.GROSS;
				disallowGrossSelection = true;
			}
		}
		logger.debug("</changeSector>" + vce.getNewValue());
	}
	
	public void changeProductCode(ValueChangeEvent vce) {
		logger.debug("<changeProductCode>" + vce.getNewValue());
		PhaseId phaseId = vce.getPhaseId();
		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			if ( vce.getNewValue() == null) {
				logger.error("Empty value for ValueChangeEvent in changeProductCode. Skipping.");
				return;
			}
			final String code = vce.getNewValue().toString();
			final ProductBenchRiskFree pbrf = productBenchManager.getPBRFWithShortDesc(code);
			if(pbrf != null)
			{	
				final Account product = pbrf.getProduct();
				final IndHdr benchmark = pbrf.getBenchmark();
				if(benchmark == null)
				{
					int benchmarkId = IndCtrlManager.getBenchmarkId(product.getId(), 1);
					IndHdr bench = benchmarkManager.findBenchmark(benchmarkId);
					pbrf.setBenchmark(bench);
					
				}
				else
					pbrf.setLeadBenchmark(benchmark);
			}
			curProdBenchRF = pbrf;
			logger.debug(curProdBenchRF);
		}		
		logger.debug("</changeProductCode>");
	}

	public void changeProduct(ValueChangeEvent vce) {
		logger.debug("<changeProduct>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			if ( vce.getNewValue() == null) {
				logger.error("Empty value for ValueChangeEvent in changeProduct. Skipping.");
				return;
			}
			final String name = vce.getNewValue().toString();
			final ProductBenchRiskFree pbrf = productBenchManager.getPBRFWithLongDesc(name);
			if(pbrf != null)
			{
				final Account product = pbrf.getProduct();
				final IndHdr benchmark = pbrf.getBenchmark();
				if(benchmark == null)
				{
					int benchmarkId = IndCtrlManager.getBenchmarkId(product.getId(), 1);
					IndHdr bench = benchmarkManager.findBenchmark(benchmarkId);
					pbrf.setBenchmark(bench);
					
				}
				else
					pbrf.setLeadBenchmark(benchmark);
			}
			curProdBenchRF = pbrf;
			logger.debug(curProdBenchRF);
		}
		logger.debug("</changeProduct>");
	}
	
	public void selectAll(ValueChangeEvent vce) {
		logger.debug("<selectAll>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			final String name = vce.getNewValue().toString();
			prodCheckedAll = new Boolean(name);
			int size = prodCheckedList.size();
			prodCheckedList.removeAllElements();
			for(int i = 0; i < size; i++) 
			{
				prodCheckedList.add(new Boolean(name));
			}
			
		}
		logger.debug("</selectAll>");
	}
	
	public void benchReqChange(ValueChangeEvent vce) {
		logger.debug("<changeBenchmarkRequired>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			final String name = vce.getNewValue().toString();
			if(name.equals("NO"))
			{
				disallowGroupbyBenchmark = true;
				groupbyBenchmark = GroupbyBenchmark.NO;
			}
			else{
				disallowGroupbyBenchmark = false;
				groupbyBenchmark = GroupbyBenchmark.YES;
			}
			if(isOnlyBenchmarks) {
				disallowGroupbyBenchmark = true;
				groupbyBenchmark = GroupbyBenchmark.YES;
			}
		}
		logger.debug("</changeBenchmarkRequired>");
	}
	
	
	public void changeSource(ValueChangeEvent vce) {
		logger.debug("<changeSource>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			source = (Source)vce.getNewValue();
			logger.debug(source.toString());
			String sourceCode = source.getSourceCode();

			productBenchList = productBenchManager.getAccountsForSource(sourceCode);
			initShortNameProductBenchList();
			curProdBenchRF = new ProductBenchRiskFree();
		}		
		
		logger.debug("</changeSource>");
	}
	
	public void changeDisplayType(ValueChangeEvent vce) {
		logger.debug("<changeDisplayType>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			displayType = (DisplayType)vce.getNewValue();
			logger.info(displayType.toString());
			String sourceCode = displayType.getSourceCode();
			String accountTypeCode = displayType.getAccountType();			

			productBenchList = productBenchManager.getAccountsForDisplayType(sourceCode, accountTypeCode);
			initShortNameProductBenchList();
			curProdBenchRF = new ProductBenchRiskFree();
		}		
		
		logger.debug("</changeDisplayType>");
	}
	
	public void changeTemplateOwner(ValueChangeEvent vce) {
		logger.debug("<changeTemplateOwner>" + vce.getNewValue());

		PhaseId phaseId = vce.getPhaseId();
		final String tempOwner = (String)vce.getNewValue();

		if ( phaseId.equals(PhaseId.ANY_PHASE))
		{
			vce.setPhaseId(PhaseId.UPDATE_MODEL_VALUES);
			vce.queue();
		}
		else if ( phaseId.equals(PhaseId.UPDATE_MODEL_VALUES))
		{
			templateOwner = tempOwner;
			initTemplatesList(templateOwner);
		}
		logger.debug("</changeTemplateOwner>" + vce.getNewValue());
	}
	
	public void setAsOf() {
		if (asOfMonth == null || asOfYear == null) {
			return;
		}
		final GregorianCalendar gc = new GregorianCalendar(Integer.parseInt(asOfYear), 
				monthLookup.get(asOfMonth), 1);
		gc.add(Calendar.MONTH, 1);//Scroll by 1 month since as of Jan should include jan figures
		curTimePeriod.asOf = gc;
	}
	
	public String addProduct() throws CloneNotSupportedException {
		if ( curProdBenchRF == null ) {
			showMessage("prodBenchRiskFree:selectedProducts", "Please select product / benchmark to be added", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		final ProductBenchRiskFree clonePBRF = (ProductBenchRiskFree)curProdBenchRF.clone();
		if ( clonePBRF == null || (clonePBRF.getProduct() == null && clonePBRF.getLeadBenchmark() == null) || clonePBRF.getBenchmark() == null )
		{
			showMessage("prodBenchRiskFree:selectedProducts", "Please select product / benchmark to be added", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		for(final ProductBenchRiskFree curPBRF : prodBenchRFs) {
			if (curPBRF.equals(curProdBenchRF)) {
				return "success";
			}
			if (curPBRF.equalsProdBench(curProdBenchRF) && curPBRF.getRiskFree() == null) {
				curPBRF.setRiskFree(curProdBenchRF.getRiskFree());
				return "success";
			}
		}
		prodBenchRFs.add(clonePBRF);
		prodCheckedList.add(false);
		prodCheckedAll = false;
		enableSectorAndMktReturns();
		//productBenchList = productBenchManager.getAccountsForDisplayType(getDisplayType().getSourceCode(), getDisplayType().getAccountType());
		return "success";
	}

	private void enableSectorAndMktReturns() {
		disallowSectorSelection = false;
		disallowGrossSelection = false;
		disallowPeriodicitySelection = false;
		allowMktSelection = false;
		allowFundAndPlanTypeSelection = false;
		disallowGroupbyBenchmark = false;
		final boolean isBenchRequired = getBenchmarkRequired().getFlag().equalsIgnoreCase("Y")?true:false;
		
		if(prodBenchRFs.size() == 0) 
			isOnlyBenchmarks = false;
		else
			isOnlyBenchmarks = true;
		
		for(final ProductBenchRiskFree curPBRF : prodBenchRFs) {
			final Account acct = curPBRF.getProduct();
			if(acct != null)
			{
				final AccountId acctID = acct.getId();
				final Source src = acctID.getSource();
				final String sourceCode = src.getSourceCode();
				
				if(acct.isChicago())
					allowFundAndPlanTypeSelection = true;
				
				if ( Source.CODE_MFP.equals(sourceCode)  ) {
					disallowSectorSelection = true;
					disallowGrossSelection = true;
					disallowPeriodicitySelection = true;
					sector = SectorEnum.TotalPortfolio;
					grossnet = GrossNet.NET;
					if (acct.isMFPCEF()) {
						allowMktSelection = true;
					}
				}
				else if ( Source.CODE_MAG.equals(sourceCode) ) {
					disallowSectorSelection = true;
					sector = SectorEnum.TotalPortfolio;
				}
				isOnlyBenchmarks = false;
			}
			else if(curPBRF.getLeadBenchmark() != null)
			{
				//disallowGrossSelection = true;
				grossnet = GrossNet.NET;
				disallowSectorSelection = true;
				sector = SectorEnum.TotalPortfolio;
			}
		}
		if (!allowMktSelection)
			returnType = "NAV";
		if(!allowFundAndPlanTypeSelection){
			fundType = false;
			planType = false;
		}
		if(isBenchRequired)
		{
			disallowGroupbyBenchmark = false;
			groupbyBenchmark = getGroupbyBenchmark();
		}
		else
		{
			disallowGroupbyBenchmark = true;
			groupbyBenchmark = GroupbyBenchmark.NO;
		}
		if(isOnlyBenchmarks) {
			disallowGroupbyBenchmark = true;
			groupbyBenchmark = GroupbyBenchmark.YES;
		}
		if ( disallowPeriodicitySelection ) // Only allow quarterly periodicity, if there is MAG account
		{
			curPeriodicity = Periodicity.MONTHLY; // To make sure that we have everything refreshed
			curTimePeriod.periodicity = Periodicity.MONTHLY;
		}
			
	}
	
	public String removeProduct() {
		final Vector<Integer> removeProdIdxs = new Vector<Integer>();
		for(int i = 0; i < prodCheckedList.size(); i++) {
			if (prodCheckedList.get(i)) {
				removeProdIdxs.add(i);
			}
		}// Which ones to remove
		int nRemovedOffset = 0;
		for(int i = 0; i < removeProdIdxs.size(); i++) {
			final int curIdx = removeProdIdxs.get(i) - nRemovedOffset;
			prodBenchRFs.remove(curIdx);
			prodCheckedList.remove(curIdx);
			nRemovedOffset++;
		}
		enableSectorAndMktReturns();
		prodCheckedAll = false;
		//productBenchList = productBenchManager.getAccountsForDisplayType(getDisplayType().getSourceCode(), getDisplayType().getAccountType());;
		return "success";
	}
	
	public String genReturnsWorkBook() {
		//Returns_ + #{qtBeanParam.retRandNum} + .xls
		hasExcelData = false;
		setInputFromState();
		// Validate data
		final String errMsg = checkPBRF();
		if (null != errMsg) {
			showMessage("prodBenchRiskFree:selectedProducts", errMsg, FacesMessage.SEVERITY_ERROR);			
			return "error";
		}
		
		final Vector<ReturnVectorData> rets = statProc.getReturns(this);
		if (null == rets || rets.size() == 0) {
			showMessage("piPerformanceReturnsForm:calendarAsOfYear", "Data is not available for the selected period", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		//ReturnVectorData.writeExcelReturns(rets, retRandNum);
		String disclosureText = getDisclosure();
		disclosureText = disclosureText.replaceAll("<br/>", "");
		ReturnVectorData.writeExcelReturns(rets, retRandNum, disclosureText);
		hasExcelData = true;
		return "success";
	}
	
	public String processCommonInput() {
		if (null != reportType && "P".equals(reportType)) {
			return processMultTrailInput();
		}
		return processInput();
	}
	
	public String processInput() {
		logger.debug(this);
		curPeriodicity = curTimePeriod.getPeriodicity();
		final String periodString = (curPeriodicity == Periodicity.MONTHLY ?"Monthly": "Quarterly");
		final String repType = periodString + " Risk Statistics";

		if (!process(repType)) {
			return "error";
		}
		return "success";
	}

	public String processMultTrailInput() {
		logger.debug(this);
		curPeriodicity = curTimePeriod.getPeriodicity();
		final String repType = "Performance Statistics";
		
		if (!process(repType)) {
			return "error";
		}
		return "success";
	}

	private boolean process(final String repType) {
		lastProcessIndex = -1;
		setInputFromState();
		// Validate data
		if ( !isValidInput()) {
			return false;
		}

		final StatisticsOutputData retData = statProc.calculateMultTrailStatistics(this);
		
		retData.setDigitsDecimal(getDigitsDecimal());
		if ( repType == "Performance Statistics") 
		{
			retData.copyTimePeriods(getTimePeriods());
		}
		else {
			retData.copyTimePeriods(statProc.getOutputPeriods());
		}
		retData.copyStatistics(getStatistics());
		retData.setSector(getSector());
		String repHdrMsg = repType + statProc.getReportHeaderMessage();
		final List<StatisticsValues> statVals = retData.getStatVals();
		if (0 == statVals.size()) {
			repHdrMsg = repHdrMsg + " : No data available";
		}

		retData.setReportHeader(repHdrMsg);
		// Work around for EL not allowing List derived class to use its members
		retData.initProdBenchNames();
		if ( repType == "Performance Statistics")
			retData.initHeaderTimePeriods("P");
		else {
			retData.initHeaderTimePeriods("R");
			retData.initInputTimePeriods(statProc.getOutputPeriods());
		}
		retData.setSortPeriod(sortPeriod);
		retData.setSortStatistic(sortStatistic);
		retData.setSortOrder(sortorder);
		boolean isGroupby = getGroupbyBenchmark().getFlag().equalsIgnoreCase("N")?false:true;
		final boolean isBenchRequired = getBenchmarkRequired().getFlag().equalsIgnoreCase("Y")?true:false;
		
		if ( sortPeriod != null && sortStatistic != null)
		{
			retData.setSortIndex();
			if ( !retData.sortStatValues(isGroupby,isBenchRequired))
			{
				showMessage("piPerformanceReturnsForm:sortingPanel", "Incorrect sorting parameters", FacesMessage.SEVERITY_ERROR);
				return false; 
			}
		}
		
		//Add Disclosures to StatisticsOutputData
		retData.setDisclosureText(getDisclosure());
		
		// TODO : add else case, when we should present data in the order of input PBRF
		allRetData.add(retData);
		lastProcessIndex = allRetData.size() - 1;
		return true;
	}
	
	private void setInputFromState() {
		setAsOf();
		timePeriods.clear();
		if (PeriodType.TRAILING == curTimePeriod.getPeriodType()) {
			for(TrailingPeriods curTP : selectedTrailingPeriods) {
				final TimePeriod tp = new TimePeriod(curTimePeriod);
				tp.setTrailingPeriod(curTP);
				timePeriods.add(tp);
			}
			if (selectedTrailingPeriods.isEmpty()) {
				timePeriods.add(new TimePeriod(curTimePeriod));
			}
		} else if (PeriodType.CALENDAR == curTimePeriod.getPeriodType()) {
			final int iFrom = Integer.parseInt(curTimePeriod.period.getFrom());
			final int iTo = Integer.parseInt(curTimePeriod.period.getTo());
			for(int i = iFrom; i <= iTo; i++) {
				final TimePeriod tp = new TimePeriod(curTimePeriod);
				tp.setPeriod(new CalendarPeriod(Integer.toString(i)));
				timePeriods.add(tp);
			}
		} else if (PeriodType.CUSTOM == curTimePeriod.getPeriodType()) {
			// There is always only on period for custom period and we could use current front-end.
			final TimePeriod tp = new TimePeriod(curTimePeriod);
			timePeriods.add(tp);
		} 
	}
	
	private boolean isValidInput()
	{
		boolean validInput = true;
		String errMessage = checkPBRF();
		String component = "prodBenchRiskFree:selectedProducts";
		if (null == errMessage) {
			errMessage = "No errors";
		} else {
			validInput = false;
			showMessage(component, errMessage, FacesMessage.SEVERITY_ERROR);
			return false;
		}
		
		final List<Statistic> lstStats = this.getStatistics();
		final List<ProductBenchRiskFree> lstPBRF = this.getProdBenchRFs();
		if ( lstStats.isEmpty())
		{
			errMessage = "No statistics were requested. Please make a selection.";
			component = "piPerformanceReturnsForm:selectStatistics";
			validInput = false;
		}
		else
		{
			boolean riskFreeStat = false;
			for (final Statistic statVal : lstStats) {
				if ( riskFreeStatistics.contains(statVal))
				{
					riskFreeStat = true;
					break;
				}
			}
			for (final ProductBenchRiskFree curPBRF : lstPBRF)
			{
				final IndHdr riskF = curPBRF.getRiskFree();
				
				if ( riskFreeStat && riskF == null)
				{
					errMessage = "Risk free rate must be present for all products for selected statistic(s).";
					component = "prodBenchRiskFree:selectedProducts";
					validInput = false;
					break;
				}
				
				// Check, if we requested SI returns for product with inception date after AsOfDate
				if ( this.selectedTrailingPeriods.contains(TrailingPeriods.SinceInception))
				{
					if(curPBRF.getProduct() != null)
					{
						Date inceptionDate = curPBRF.getProduct().getInceptionDate();
						if ( inceptionDate.after(curTimePeriod.getAsOf().getTime()))
						{
							final SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy");
							errMessage = "Selected product " + curPBRF.getProduct().getLongName()+ " has inception date " + 
									sdf.format(inceptionDate) +
									" after As Of Date selected. Please change date or remove the product or SI period.";
							component = "prodBenchRiskFree:selectedProducts";
							validInput = false;
							break;
						}
					}
					else if(curPBRF.getLeadBenchmark() != null) 
					{
						errMessage = "Inception Dates are not available for Benchmarks.";
						validInput = false;
						break;
					}					
				}
			}
			
			if ( (sortPeriod != null && sortStatistic == null) ||
				 (sortPeriod == null && sortStatistic != null) )
			{
				errMessage = "Both sorting parameters (period and statistic) should be provided.";
				component = "piPerformanceReturnsForm:sortingPanel";
				validInput = false;
			}
			else {
				if ( sortPeriod != null && !selectedTrailingPeriods.contains(sortPeriod) )
				{
					errMessage = "Results can be sorted only by selected trailing period.";
					component = "piPerformanceReturnsForm:sortingPanel";
					validInput = false;
				}
				
				if ( sortStatistic != null && !statistics.contains(sortStatistic) )
				{
					errMessage = "Results can be sorted only by selected trailing statistic.";
					component = "piPerformanceReturnsForm:sortingPanel";
					validInput = false;
				}
			}
			
			// Check, if someone was stupid enough to request MRM returns with Quarterly periodicity
			if ( this.selectedTrailingPeriods.contains(TrailingPeriods.MostRecentMonth) &&
					curTimePeriod.periodicity.equals(Periodicity.QUARTERLY) )
			{
				errMessage = "You could not request MRM performance with quarterly periodicity.";
				component = "piPerformanceReturnsForm:periodicity";
				validInput = false;
			}
		}
		
		if ( !validInput )
		{
			showMessage(component, errMessage, FacesMessage.SEVERITY_ERROR);
			return false;
		}
		return true;
	}
	
	private String checkPBRF() {
		String errMessage = null;
		boolean bContainsSI = false;
		final List<TimePeriod> lstTPs = getTimePeriods();
		for (final TimePeriod curTP : lstTPs) {
			if (PeriodType.TRAILING == curTP.getPeriodType() &&
				TrailingPeriods.SinceInception == curTP.getTrailingPeriod()) {
				bContainsSI = true;
				break;
			}
		}
		final List<ProductBenchRiskFree> lstPBRF = this.getProdBenchRFs();
		if ( lstPBRF.isEmpty()) {
			errMessage = "No products were requested. Please make a selection.";
		} else {
			for (final ProductBenchRiskFree curPBRF : lstPBRF)
			{
				final Account prod = curPBRF.getProduct();
				final IndHdr leadBench = curPBRF.getLeadBenchmark();

				if ( this.selectedTrailingPeriods.contains(TrailingPeriods.SinceInception))
				{
					if(leadBench != null) 
					{
						errMessage = "Inception Dates are not available for Benchmarks.";
						break;
					}
				}
				
				if ( prod == null && leadBench == null)
				{
					errMessage = "Product / Benchmark is not selected.";
					break;
				}
				else if (bContainsSI &&
						Source.CODE_CMB.equals(prod.getId().getSource().getSourceCode())) {
					errMessage = "Combined Portfolio with Since Inception NOT allowed.";
					break;
				}
			}
		}
		
		return errMessage;
	}
	
	private void showMessage (final String component, final String errMessage, FacesMessage.Severity severity)
	{
		final FacesContext facesContext = FacesContext.getCurrentInstance();
		final FacesMessage message = new FacesMessage();
		if ( severity == null )
			severity = FacesMessage.SEVERITY_ERROR;
		message.setSeverity(severity);			
		message.setSummary(errMessage);
		message.setDetail(errMessage);
		facesContext.addMessage(component, message);
	}

	public List<SelectItem> getShortNameProductBenchList() {
		if (null == shortNameProductBenchList) {
			getProductBenchList();
			initShortNameProductBenchList();
		}
		return shortNameProductBenchList;
	}
	
	private List<SelectItem> initTemplatesList(final String user)
	{
		templateList = new Vector<SelectItem>();
		templateMgr.init();
		final HashMap<String, HashMap<String, Template>> templateMap = templateMgr.getTemplatesHash();
		final HashMap<String, Template> userTemplateMap = templateMap.get(user);
		
		if ( userTemplateMap == null || userTemplateMap.isEmpty())
			return null;
		
		final Vector<Template> templateVector = new Vector<Template>();
		for (final Template curTP : userTemplateMap.values())
		{
			templateVector.add(curTP);
		}
		Collections.sort(templateVector);
		
		for (final Template curTP : templateVector)
		{
			final String tempName = new String(curTP.getTemplateName());
			templateList.add(new SelectItem(tempName, tempName));
		}
		if ( templateVector.size()>0 )
			templateName = templateVector.get(0).getTemplateName();
		else
			templateName = new String();
		return templateList;
	}
	
	private void initTemplateOwnersList() {
		templateOwnersList = new Vector<SelectItem>(); 
		final List<String> owners = templateMgr.getTemplateOwners();
		templateOwner = "";
		for ( final String curOwner : owners)
		{
			templateOwnersList.add(new SelectItem(curOwner, curOwner));
			// Set template owner to the user, if he has any templates
			if ( curOwner.equals(usrSecurityBean.getUserID()))
				templateOwner = usrSecurityBean.getUserID();
		}
	}
	
	private boolean isTemplateReady() {
		if ( templateName == null || templateName.equals(""))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "Template name should be entered", FacesMessage.SEVERITY_ERROR);
			return false;
		}
		// Check parameters
		if ( prodBenchRFs.isEmpty()) {
			showMessage("piPerformanceReturnsForm:templateParams", "No products were requested. Please make a selection.", FacesMessage.SEVERITY_ERROR);
			return false;
		}
		if ( statistics.isEmpty())
		{
			showMessage("piPerformanceReturnsForm:templateParams", "No statistics were requested. Please make a selection.", FacesMessage.SEVERITY_ERROR);
			return false;
		}
		if ( curTimePeriod.periodType == PeriodType.TRAILING )
		{
			if ( selectedTrailingPeriods.size() == 0 ) {
				showMessage("piPerformanceReturnsForm:templateParams", "No trailing periods were requested. Please make a selection.", FacesMessage.SEVERITY_ERROR);
				return false;
			}
		}
		else if ( curTimePeriod.periodType == PeriodType.CALENDAR )
		{
			if ( curTimePeriod.period.getFrom() == null || curTimePeriod.period.getTo() == null)
			{
				showMessage("piPerformanceReturnsForm:templateParams", "Please select calendar period.", FacesMessage.SEVERITY_ERROR);
				return false;
			}
		}
		else if ( curTimePeriod.periodType == PeriodType.CUSTOM )
		{
			if ( curTimePeriod.custPeriod.getFrom() == null || curTimePeriod.custPeriod.getTo() == null)
			{
				showMessage("piPerformanceReturnsForm:templateParams", "Please select custom period.", FacesMessage.SEVERITY_ERROR);
				return false;
			}
		}
		// Everything looks good, OK to proceed with saving the template
		return true;
	}
	
	public String loadTemplate()
	{
		if ( templateName == null || templateName.equals(""))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "Template name should be selected", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		if ( templateOwner == null || templateOwner.equals(""))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "Template owner should be selected", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		
		final Template loadTemplate = templateMgr.getTemplate(templateName, templateOwner);
		
		if ( loadTemplate == null)
		{
			showMessage("piPerformanceReturnsForm:templateParams", 
						"Template "+templateName+" created by "+templateOwner+" was not found.", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		
		final List<TemplateProducts> templateProducts = TemplateManager.getTemplateProducts(loadTemplate);
		final List<TemplatePeriods> templatePeriods = TemplateManager.getTemplatePeriods(loadTemplate);
		final List<TemplateStats> templateStats = TemplateManager.getTemplateStats(loadTemplate);
		
		// Clear and re-populate list of products
		prodBenchRFs.clear();
		prodCheckedList.clear();
		prodCheckedAll = false;
		
		HashMap<AccountId, Account> idHashMap = productBenchManager.getIDHash();
		
		for (final TemplateProducts tempProduct : templateProducts) {
			final ProductBenchRiskFree tempPBRF = new ProductBenchRiskFree();
			
			Integer leadNo = tempProduct.getLeadBenchmarkNo();
			if ( leadNo == null )
			{
				final AccountId actid = new AccountId(tempProduct.getSourceCode(), tempProduct.getAccountCode());
				final Account acct = idHashMap.get(actid);
				if(acct == null) 
				{
					logger.error("Problem getting Account for account id "+ actid +". Please Investigate.");
					continue;
				}
				String code = acct.getShortName();
				if ( code==null)
					code = acct.getDummyName();
				tempPBRF.setCode(code);
				tempPBRF.setName(acct.getLongName());
				tempPBRF.setProduct(acct);
				tempPBRF.setShortNameProduct(acct);
			}
			else
			{
				final IndHdr leadBenchmark = benchmarkManager.findBenchmark(leadNo);
				if(leadBenchmark == null)
				{
					logger.error("Problem getting Lead Benchmark for benchmark id "+ leadNo +". Please Investigate.");
					continue;
					
				}
				tempPBRF.setCode(leadBenchmark.getShortDesc());
				tempPBRF.setName(leadBenchmark.getFullDesc());
				tempPBRF.setLeadBenchmark(leadBenchmark);
			}
			
			final IndHdr benchmark = benchmarkManager.findBenchmark(tempProduct.getBenchmarkNo());
			final Integer riskfreeNo = tempProduct.getRiskfreeNo();
			if(benchmark != null)
			{
				tempPBRF.setBenchmark(benchmark);
				if ( riskfreeNo != null)
				{
					final IndHdr riskfree = benchmarkManager.findBenchmark(tempProduct.getRiskfreeNo());
					tempPBRF.setRiskFree(riskfree);
				}
			}
			prodBenchRFs.add(tempPBRF);
			prodCheckedList.add(false);
			
			
		}
		if( prodBenchRFs.size() == 0 )
		{
			logger.error("Template "+templateName+" created by "+templateOwner+" has no accounts / benchmarks found.");
			showMessage("piPerformanceReturnsForm:templateParams", 
					"Template "+templateName+" created by "+templateOwner+" has no accounts / benchmarks found.", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		if( templateProducts.size() != prodBenchRFs.size() )
		{
			logger.warn("Template "+templateName+" created by "+templateOwner+" has mismatch in accounts / benchmarks while loading.");
			//TODO : Need to display WARN level message to user 
		}
		curProdBenchRF = new ProductBenchRiskFree();
		
		// Clear and re-populate list of statistics
		resetPeriodsAndStats();
		for (final TemplateStats tempStat : templateStats) {
			for (final Statistic stat : allStatistics)
			{
				if ( tempStat.getStatisticNo() == stat.ordinal())
				{
					// Update both source and target lists
					statistics.add(stat);
					if ( allStatistics.contains(stat))
						allStatistics.remove(stat);
					break;
				}
			}
		}
		
		// Clear and re-populate list of periods
		timePeriods.clear();
		selectedTrailingPeriods.clear();
		for ( final TemplatePeriods tempPeriod : templatePeriods) {
			if (tempPeriod.getPeriodType().equals("TRAILING"))
			{
				curTimePeriod.periodType = PeriodType.TRAILING;	
				trailingVisible = true;
				calendarVisible = false;
				customVisible = false;
				allowSorting = true;
				for (final TrailingPeriods trail : TrailingPeriods.values()) 
				{
					if ( tempPeriod.getTrailingNo() == trail.ordinal())
					{
						// Update both source and target lists
						selectedTrailingPeriods.add(trail);
						if ( allTrailingPeriods.contains(trail))
							allTrailingPeriods.remove(trail);
						break;
					}
				}
			}
			else if (tempPeriod.getPeriodType().equals("CALENDAR"))
			{
				curTimePeriod.periodType = PeriodType.CALENDAR;
				calendarVisible = true;
				trailingVisible = false;
				customVisible = false;
				allowSorting = false;
				if (tempPeriod.getCalendarFrom() != null && tempPeriod.getCalendarTo() != null)
				{
					curTimePeriod.period.setFrom(tempPeriod.getCalendarFrom().toString());
					curTimePeriod.period.setTo(tempPeriod.getCalendarTo().toString());
				}
			}
			else if (tempPeriod.getPeriodType().equals("CUSTOM"))
			{
				curTimePeriod.periodType = PeriodType.CUSTOM;
				customVisible = true;
				trailingVisible = false;
				customVisible = true;
				allowSorting = false;
				if (tempPeriod.getCustomFrom() != null && tempPeriod.getCustomTo() != null)
				{
					curTimePeriod.custPeriod.setFrom(tempPeriod.getCustomFrom());
					curTimePeriod.custPeriod.setTo(tempPeriod.getCustomTo());
				}
			}
		}
		
		// Reset other request parameters
		// Periodicity could be quarterly only for performance screen
		if ( null != reportType && "P".equals(reportType) 
				&& loadTemplate.getPeriodicity().equals("Q"))
			curTimePeriod.periodicity=Periodicity.QUARTERLY;
		else
			curTimePeriod.periodicity=Periodicity.MONTHLY;
		
		if ( loadTemplate.getGrossNet().equals("G"))
			grossnet = GrossNet.GROSS;
		else
			grossnet = GrossNet.NET;
		
		for ( final SectorEnum sect : SectorEnum.values())
		{
			if (loadTemplate.getSectorNo() == sect.getSectorID())
			{
				sector = sect;
				break;
			}
		}
		
		final Integer sortpd = loadTemplate.getSortPeriod();
		if ( sortpd != null) {
			for (final TrailingPeriods trail : TrailingPeriods.values())
			{				
				if (sortpd  == trail.ordinal())
				{
					sortPeriod = trail;
					break;
				}
			}
		}
		else
			sortPeriod = null;
		
		final Integer sortstat = loadTemplate.getSortStatistic();
		if ( sortstat != null) {
			for (final Statistic stat : Statistic.values())
			{
				if ( sortstat == stat.ordinal())
				{
					sortStatistic = stat;
					break;
				}
			}
		}
		else
			sortStatistic = null;
		
		final String groupbyBM = loadTemplate.getGroupbyBenchmark();
		if(groupbyBM != null && (groupbyBM.equalsIgnoreCase(GroupbyBenchmark.NO.getFlag())))
			groupbyBenchmark = GroupbyBenchmark.NO;
		else
			groupbyBenchmark = GroupbyBenchmark.YES; 
		
		final String ignoreBM = loadTemplate.getBenchmarkRequired();
		if(ignoreBM != null && (ignoreBM.equalsIgnoreCase(BenchmarkRequired.NO.getFlag())))
			benchmarkRequired = BenchmarkRequired.NO;
		else
		{
			benchmarkRequired = BenchmarkRequired.YES;
		}
		
		enableSectorAndMktReturns();
		
		return cleanReRender();
	}
	
	public String saveTemplate() 
	{
		final String saveTemplateName = templateName;
		final String user = usrSecurityBean.getUserID();
		if ( isTemplateReady() )
		{
			// Parameters for the template
			// Current date
			Date curDate = new Date();
			
			String grossNet = "N";
			if ( grossnet == GrossNet.GROSS)
				grossNet = "G";
			
			String periodicity = "M";
			if ( curTimePeriod.periodicity == Periodicity.QUARTERLY)
				periodicity = "Q";
			
			// Sector will take as is - sector
			Integer sect = new Integer(sector.getSectorID());
 
			Integer sortStat = null;
			if ( sortStatistic != null)
				sortStat = new Integer(sortStatistic.ordinal());
			
			Integer sortPd = null;
			if ( sortPeriod != null)
				sortPd = new Integer(sortPeriod.ordinal());
			
			final Template curTemplate = new Template(templateName, user, curDate,
					grossNet, periodicity, sect, sortPd, sortStat, groupbyBenchmark.getFlag(), benchmarkRequired.getFlag());
			
			final Vector<TemplateProducts> saveTemProducts = new Vector<TemplateProducts>();
			final Vector<TemplatePeriods> saveTemPeriods = new Vector<TemplatePeriods>();
			final Vector<TemplateStats> saveTemStats = new Vector<TemplateStats>();
			
			Template origTemplate = templateMgr.getTemplate(templateName, user);
			if ( origTemplate == null) // New template
			{
				TemplateManager.save(curTemplate);	
				
				populateTemplateParams(curTemplate, saveTemProducts,
						saveTemPeriods, saveTemStats);
				
				// Save template products
				TemplateManager.saveTemplateProducts(saveTemProducts);
				
				// Save template periods
				TemplateManager.saveTemplatePeriods(saveTemPeriods);
				
				// Save template stats
				TemplateManager.saveTemplateStats(saveTemStats);
			}
			else // Update current template
			{
				if ( origTemplate.update(curTemplate, user))
					TemplateManager.update(origTemplate);
				
				populateTemplateParams(origTemplate, saveTemProducts,
						saveTemPeriods, saveTemStats);
				
				// Re-populate (delete and insert) template params 
				// (products, periods and stats)
				TemplateManager.updateTemplateProducts(origTemplate, saveTemProducts);
				TemplateManager.updateTemplatePeriods(origTemplate, saveTemPeriods);
				TemplateManager.updateTemplateStats(origTemplate, saveTemStats);
			}

			initTemplates();
			
			showMessage("piPerformanceReturnsForm:templateParams", "Template has been saved successfully", FacesMessage.SEVERITY_INFO);
			
			templateName = saveTemplateName;
			
			return "success";
		}
		else
			return "error";
		
	}
	
	public String deleteTemplate()
	{
		final String user = usrSecurityBean.getUserID();
		if ( templateName == null || templateName.equals(""))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "Template name should be selected.", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		if ( !templateOwner.equals(user))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "You could only delete your own templates.", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		
		final HashMap<String, HashMap<String, Template>> templateMap = templateMgr.getTemplatesHash();
		final HashMap<String, Template> userTemplateMap = templateMap.get(user);
		
		if ( userTemplateMap == null || userTemplateMap.isEmpty() || 
					!userTemplateMap.containsKey(templateName))
		{
			showMessage("piPerformanceReturnsForm:templateParams", "Template " + templateName + " not found.", FacesMessage.SEVERITY_ERROR);
			return "error";
		}
		
		final Template template = userTemplateMap.get(templateName);
		
		TemplateManager.deleteTemplate(template);
		
		initTemplates();	
		
		showMessage("piPerformanceReturnsForm:templateParams", "Template has been deleted successfully", FacesMessage.SEVERITY_INFO);
		
		templateName = "";
		
		return "success";
	}

	private void populateTemplateParams(final Template template,
			final Vector<TemplateProducts> saveTemProducts,
			final Vector<TemplatePeriods> saveTemPeriods,
			final Vector<TemplateStats> saveTemStats) 
	{
		int count = 1;
		for ( final ProductBenchRiskFree curPBRF : prodBenchRFs)
		{
			final TemplateProductsId tpID = new TemplateProductsId(template.getTemplateId(), count++);
			//final AccountId acctID = curPBRF.getProduct().getId();
			String sourceCode = null;
			String accountCode = null;
			final Account acct = curPBRF.getProduct();
			AccountId acctID = null;
			IndHdr leadBenchHdr = null;
			if(acct != null){
				acctID = acct.getId();
				sourceCode = acctID.getSource().getSourceCode();
				accountCode = acctID.getAccountCode();
			}
			else
			{
				leadBenchHdr = curPBRF.getLeadBenchmark();
				sourceCode = "PMR";
			}
			Integer leadBenchmarkNo = null;
			if ( leadBenchHdr != null)
				leadBenchmarkNo = leadBenchHdr.getIndHdrNo();
			final IndHdr benchHdr = curPBRF.getBenchmark();
			Integer benchmarkNo = null;
			if ( benchHdr != null)
				benchmarkNo = benchHdr.getIndHdrNo();
			final IndHdr riskFreeHdr = curPBRF.getRiskFree();
			Integer riskFreeNo = null;
			if ( riskFreeHdr != null)
				riskFreeNo = riskFreeHdr.getIndHdrNo();
			/*final TemplateProducts tp = new TemplateProducts(tpID, acctID.getSource().getSourceCode(),
								acctID.getAccountCode(), benchmarkNo, riskFreeNo);*/
			final TemplateProducts tp = new TemplateProducts(tpID, sourceCode, accountCode, 
					benchmarkNo, riskFreeNo, leadBenchmarkNo);
			saveTemProducts.add(tp);
		}
		
		if ( curTimePeriod.periodType == PeriodType.TRAILING)
		{
			count = 1;
			for(final TrailingPeriods curTP : selectedTrailingPeriods) {
				final TemplatePeriodsId tpID = new TemplatePeriodsId(template.getTemplateId(), count++);
				final TemplatePeriods tp = new TemplatePeriods(tpID, "TRAILING", curTP.ordinal(), null, null, null, null);
				saveTemPeriods.add(tp);
			}
		}
		else if ( curTimePeriod.periodType == PeriodType.CALENDAR)
		{
			final int iFrom = Integer.parseInt(curTimePeriod.period.getFrom());
			final int iTo = Integer.parseInt(curTimePeriod.period.getTo());
			final TemplatePeriodsId tpID = new TemplatePeriodsId(template.getTemplateId(), 1);
			final TemplatePeriods tp = new TemplatePeriods(tpID, "CALENDAR", null, iFrom, iTo, null, null);
			saveTemPeriods.add(tp);
		}
		else // Custom period
		{
			final TemplatePeriodsId tpID = new TemplatePeriodsId(template.getTemplateId(), 1);
			final TemplatePeriods tp = new TemplatePeriods(tpID, "CUSTOM", null, null, null, 
									curTimePeriod.custPeriod.getFrom(), curTimePeriod.custPeriod.getTo());
			saveTemPeriods.add(tp);
		}
		
		count = 1;
		for ( final Statistic curStat : statistics)
		{
			final TemplateStatsId tsID = new TemplateStatsId(template.getTemplateId(), count++);
			final TemplateStats ts = new TemplateStats(tsID, curStat.ordinal());
			saveTemStats.add(ts);
		}
	}
	
	private String cleanReRender() {
		final ExternalContext ectx = FacesContext.getCurrentInstance().getExternalContext();
		String sPageLink;
		if ( reportType == "P")
		{
			sPageLink = "piQTPerformanceReturns.faces";
		}
		else
		{
			sPageLink = "piQueryTool.faces";
		}
		final HttpServletResponse response = (HttpServletResponse)ectx.getResponse();
		try {
			response.sendRedirect(sPageLink);
		} catch (IOException e) {
			logger.error("Problem redirect to original page. Please contact IT support.");
		}
		return sPageLink;
	}
	
	private void initShortNameProductBenchList() {
		shortNameProductBenchList = new Vector<SelectItem>();
		HashMap<String, ProductBenchRiskFree> origHash = null;
		final String displayTypeCode = displayType.getDisplayTypeCode();
		final String sourceCode = displayType.getSourceCode();
		final String accountTypeCode = displayType.getAccountType();
		final String[] typeArray = accountTypeCode != null ? accountTypeCode.split("\\|") : null;
		final List<String> typeList = typeArray != null ? Arrays.asList(typeArray) : null;
		if(displayTypeCode.equalsIgnoreCase("PMR")) 
			origHash = productBenchManager.getBenchHash();
		else
			origHash = productBenchManager.getHash();
		
		final Set<String> kSet = origHash.keySet();
		for(final String keyOrig : kSet) {
			ProductBenchRiskFree pbrf = origHash.get(keyOrig);
			final Account account = pbrf.getProduct();
			final IndHdr hdr = pbrf.getBenchmark();
			if(account == null) {
				String key = hdr.getShortDesc().trim();
				final String fullKey = key + "_" + hdr.getIndHdrNo().toString();
				if (null == fullKey || 0 == fullKey.length()) {
					continue;
				}
				shortNameProductBenchList.add(new SelectItem(pbrf, fullKey));
			}
			else {
				final Source acctSrc = account.getId().getSource();
				final String acctSrcCode = acctSrc.getSourceCode();
				final String acctType = account.getAccountType();
				
				// Source code should be always equal, account type is optional
				if (!acctSrcCode.equals(sourceCode) ||
						(accountTypeCode != null && typeList != null && !typeList.contains(acctType))) {
					continue;
				}
				String key = account.getShortName();
				if ( key==null) // Use dummy name instead
					key = account.getDummyName();
				pbrf.setCode(key);
				if (null == key || 0 == key.length()) {
					continue;
				}
				if ( !account.isCOM_MF())
					shortNameProductBenchList.add(new SelectItem(pbrf, key));
			}
		}
		Collections.sort(shortNameProductBenchList, new SelectItemComparator());
	}
	
	public List<SelectItem> getProductBenchList() {
		if (null==productBenchList) {
			productBenchList = productBenchManager.getAccountsForDisplayType(getDisplayType().getSourceCode(), getDisplayType().getAccountType());;
		}
		return productBenchList;
	}

	public void setProductBenchList(List<SelectItem> productBenchList) {
		this.productBenchList = productBenchList;
	}

	public List<SelectItem> getBenchmarkList() {
		if (null==benchmarkList) {
			benchmarkList = benchmarkManager.getBenchmarks();
		}
		
		return benchmarkList; 
	}
	
	public void setBenchmarkList(List<SelectItem> benchmarkList) {
		this.benchmarkList = benchmarkList;
	}
	
	public List<SelectItem> getRiskFreeList() {
		if (null==riskFreeList) {
			getBenchmarkList();//Initialize if required
			riskFreeList = new ArrayList<SelectItem>();
			for (final SelectItem selItem : benchmarkList) {
				final IndHdr indHdr = (IndHdr) selItem.getValue();
				if (indHdr.getRiskFree().equalsIgnoreCase("Y")) {
					riskFreeList.add(selItem);
					break;
				}
			}
		}
		return riskFreeList;
	}

	public void setRiskFreeList(List<SelectItem> riskFreeList) {
		this.riskFreeList = riskFreeList;
	}

	public AccountLongNameConverter getAccountManager() {
		return accountManager;
	}

	public void setAccountManager(AccountLongNameConverter accountManager) {
		this.accountManager = accountManager;
		statProc.setAcctMgr(accountManager);
	}
	
	public ProductBenchConverter getProductBenchManager() {
		return productBenchManager;
	}

	public void setProductBenchManager(
			ProductBenchConverter productBenchManager) {
		this.productBenchManager = productBenchManager;
		statProc.setAcctBenchMgr(productBenchManager);
	}

	public IndHdrMgrConverter getBenchmarkManager() {
		return benchmarkManager;
	}

	public void setBenchmarkManager(IndHdrMgrConverter benchmarkManager) {
		this.benchmarkManager = benchmarkManager;
		statProc.setBenchMgr(benchmarkManager);
	}
	
	public SourceConverter getSourceConverter() {
		return sourceConverter;
	}

	public void setSourceConverter(SourceConverter sourceConverter) {
		this.sourceConverter = sourceConverter;
	}

	public void setDisplayTypeConverter(DisplayTypeConverter displayTypeConverter) {
		this.displayTypeConverter = displayTypeConverter;
	}

	public DisplayTypeConverter getDisplayTypeConverter() {
		return displayTypeConverter;
	}

	public void setTemplateConverter(TemplateConverter templateConverter) {
		this.templateConverter = templateConverter;
	}

	public TemplateConverter getTemplateConverter() {
		return templateConverter;
	}

	public List<SelectItem> getPeriodList() {
		return periodList;
	}

	public void setPeriodList(List<SelectItem> periodList) {
		this.periodList = periodList;
	}

	public List<Statistic> getAllStatistics() {
		return allStatistics;
	}

	public void setAllStatistics(List<Statistic> allStatistics) {
		this.allStatistics = allStatistics;
	}

	public void setRiskFreeStatistics(List<Statistic> riskFreeStatistics) {
		this.riskFreeStatistics = riskFreeStatistics;
	}

	public List<Statistic> getRiskFreeStatistics() {
		return riskFreeStatistics;
	}

	public boolean isCalendarVisible() {
		return calendarVisible;
	}

	public void setCalendarVisible(boolean calendarVisible) {
		this.calendarVisible = calendarVisible;
	}

	public boolean getTrailingVisible() {
		return trailingVisible;
	}

	public void setTrailingVisible(boolean trailingVisible) {
		this.trailingVisible = trailingVisible;
	}

	public void setCustomVisible(boolean customVisible) {
		this.customVisible = customVisible;
	}

	public boolean isCustomVisible() {
		return customVisible;
	}

	public List<TrailingPeriods> getAllTrailingPeriods() {
		return allTrailingPeriods;
	}

	public void setAllTrailingPeriods(List<TrailingPeriods> allTrailingPeriods) {
		this.allTrailingPeriods = allTrailingPeriods;
	}

	private void resetPeriodsAndStats()
	{
		selectedTrailingPeriods = new Vector<TrailingPeriods>();
		final Vector<TrailingPeriods> tps = new Vector<TrailingPeriods>();
		tps.addAll(Arrays.asList(TrailingPeriods.values()));
		TrailingPeriods tpYTD = tps.remove(3); //REMEMBER TO UPDATE
		allTrailingPeriods = tps;
		final int curMonth = curTimePeriod.asOf.get(Calendar.MONTH);
		final int insIdx;
		if (curMonth + 1 <= 3) {
			insIdx = 1;
		} else if (curMonth + 1 <= 6){
			insIdx = 2;
		} else {
			insIdx = 3;
		}
		tps.add(insIdx, tpYTD);
		
		statistics.clear();
		if ( reportType == "P")
		{
			allStatistics = Statistic.getVisiblePerformanceStatistics();
		}
		else
		{
			allStatistics = Statistic.getVisibleStatistics();
			// Modify allTrailingPeriods to remove MRQ..3Year
			int i = 0;
			for (;i < allTrailingPeriods.size(); i++) {
				if (allTrailingPeriods.get(i) == TrailingPeriods.ThreeYear) {
					break;
				}
			}
			allTrailingPeriods = allTrailingPeriods.subList(i, allTrailingPeriods.size());
		}
	}
	
	public void setPerformStatsOnly(boolean performStatsOnly) {
		if (performStatsOnly) {
			allStatistics = Statistic.getVisiblePerformanceStatistics();
			setReportType("P");
		} else {
			setReportType("R");
			// Modify allTrailingPeriods to remove MRQ..3Year
			int i = 0;
			for (;i < allTrailingPeriods.size(); i++) {
				if (allTrailingPeriods.get(i) == TrailingPeriods.ThreeYear) {
					break;
				}
			}
			allTrailingPeriods = allTrailingPeriods.subList(i, allTrailingPeriods.size());
		}
		
		//Add Disclosures
		try{
			List<Disclosures> allDisclosures = DisclosuresManager.getDisclosuresForType(getReportType());
			for(Disclosures dis : allDisclosures) {
				String fileName = dis.getFileName();
				InputStream in = this.getClass().getClassLoader().getResourceAsStream(fileName);
				POIFSFileSystem fs = new POIFSFileSystem(in);
				HWPFDocument doc = new HWPFDocument(fs);
				WordExtractor we = new WordExtractor(doc);
				String[] paragraphs = we.getParagraphText();
				for( int index = 0; index < paragraphs.length; index++ ) {
					disclosure += paragraphs[index];
					disclosure += "<br/>";
				}
				disclosure += "<br/>";
			}
		}
		catch(Exception e) { 
			logger.error("Error while reading the disclosure documents"+e.getMessage());
		}
		
		sortStatisticList.add(new SelectItem(null,"")); //empty record, if we don't want to sort
		for ( final Statistic st : allStatistics)
		{
			sortStatisticList.add(new SelectItem(st, st.getDisplayName())); 
		}
		
		sortPeriodList.add(new SelectItem(null,"")); //empty record, if we don't want to sort
		for ( final TrailingPeriods tp : allTrailingPeriods)
		{
			sortPeriodList.add(new SelectItem(tp, tp.getDisplay()));
		}
		
		initTemplates();
	}

	private void initTemplates() {
		initTemplateOwnersList();
		initTemplatesList(templateOwner);
	}

	public String getAsOfMonth() {
		return asOfMonth;
	}

	public void setAsOfMonth(String asOfMonth) {
		this.asOfMonth = asOfMonth;
	}

	public String getAsOfYear() {
		return asOfYear;
	}

	public void setAsOfYear(String asOfYear) {
		this.asOfYear = asOfYear;
	}

	public List<SelectItem> getMonthList() {
		return monthList;
	}

	public void setMonthList(List<SelectItem> monthList) {
		this.monthList = monthList;
	}

	public String getReportType() {
		return reportType;
	}

	public void setReportType(String reportType) {
		this.reportType = reportType;
	}

	public List<TrailingPeriods> getSelectedTrailingPeriods() {
		return selectedTrailingPeriods;
	}

	public void setSelectedTrailingPeriods(
			List<TrailingPeriods> selectedTrailingPeriods) {
		this.selectedTrailingPeriods = selectedTrailingPeriods;
	}

	public void setSortPeriod(TrailingPeriods sortPeriod) {
		this.sortPeriod = sortPeriod;
	}

	public TrailingPeriods getSortPeriod() {
		return sortPeriod;
	}

	public void setSortPeriodList(List<SelectItem> sortPeriodList) {
		this.sortPeriodList = sortPeriodList;
	}

	public List<SelectItem> getSortPeriodList() {
		return sortPeriodList;
	}

	public void setSortStatistic(Statistic sortStatistic) {
		this.sortStatistic = sortStatistic;
	}

	public Statistic getSortStatistic() {
		return sortStatistic;
	}

	public void setSortStatisticList(List<SelectItem> sortStatisticList) {
		this.sortStatisticList = sortStatisticList;
	}

	public List<SelectItem> getSortStatisticList() {
		return sortStatisticList;
	}

	public void setTemplateList(List<SelectItem> templateList) {
		this.templateList = templateList;
	}

	public List<SelectItem> getTemplateList() {
		return templateList;
	}

	public void setTemplateName(String templateName) {
		this.templateName = templateName;
	}

	public String getTemplateName() {
		return templateName;
	}

	public void setTemplateOwnersList(List<SelectItem> templateOwnersList) {
		this.templateOwnersList = templateOwnersList;
	}

	public List<SelectItem> getTemplateOwnersList() {
		return templateOwnersList;
	}

	public void setTemplateOwner(String templateOwner) {
		this.templateOwner = templateOwner;
	}

	public String getTemplateOwner() {
		return templateOwner;
	}

	public ProductBenchRiskFree getCurProdBenchRF() {
		if ( curProdBenchRF == null ) 
			curProdBenchRF = new ProductBenchRiskFree();
		getRiskFreeList();
		if(riskFreeList != null && riskFreeList.size() > 0)
			curProdBenchRF.riskFree = (IndHdr) riskFreeList.get(0).getValue();
		return curProdBenchRF;
	}

	public void setCurProdBenchRF(ProductBenchRiskFree curProdBenchRF) {
		this.curProdBenchRF = curProdBenchRF;
	}

	public Vector<Boolean> getProdCheckedList() {
		return prodCheckedList;
	}

	public void setProdCheckedList(Vector<Boolean> prodCheckedList) {
		this.prodCheckedList = prodCheckedList;
	}

	public TimePeriod getCurTimePeriod() {
		return curTimePeriod;
	}

	public void setCurTimePeriod(TimePeriod curTimePeriod) {
		this.curTimePeriod = curTimePeriod;
	}

	public void setSourceList(List<SelectItem> sourceList) {
		this.sourceList = sourceList;
	}

	public List<SelectItem> getSourceList() {
		if (null==sourceList) {
			sourceList = sourceConverter.getSources();
		}
		return sourceList;
	}

	public List<SelectItem> getDisplayTypeList() {
		if (null==displayTypeList) {
			displayTypeList = displayTypeConverter.getDisplayTypes();
		}
		return displayTypeList;
	}

	public void setDisplayTypeList(List<SelectItem> displayTypeList) {
		this.displayTypeList = displayTypeList;
	}

	public void setSource(Source source) {
		this.source = source;
	}

	public Source getSource() {
		if ( source == null ) {
			final List<SelectItem> srcLst = getSourceList();
			for (final SelectItem curSI : srcLst) {
				final Source curSrc = (Source) curSI.getValue();
				if (Source.CODE_COM.equals(curSrc.getSourceCode())) {
					source = curSrc;
					break;
				}
			}
		}
		return source;
	}

	public DisplayType getDisplayType() {
		if ( displayType == null) {
			final List<SelectItem> dispTypeLst = getDisplayTypeList();
			for (final SelectItem curSI : dispTypeLst) {
				final DisplayType curDispType = (DisplayType) curSI.getValue();
				if (Source.CODE_COM.equals(curDispType.getSourceCode())) {
					displayType = curDispType;
					break;
				}
			}
		}
		return displayType;
	}

	public void setDisplayType(DisplayType displayType) {
		this.displayType = displayType;
	}

	public int getDigitsDecimal() {
		return digitsDecimal;
	}
	public void setDigitsDecimal(int digitsDecimal) {
		this.digitsDecimal = digitsDecimal;
	}
	
	public int getCurIndex() {
		return lastProcessIndex;
	}

	public Vector<StatisticsOutputData> getAllRetData() {
		return allRetData;
	}

	public boolean isDisallowSectorSelection() {
		return disallowSectorSelection;
	}

	public void setDisallowSectorSelection(boolean disallowSectorSelection) {
		this.disallowSectorSelection = disallowSectorSelection;
	}

	public void setDisallowGrossSelection(boolean disallowGrossSelection) {
		this.disallowGrossSelection = disallowGrossSelection;
	}

	public boolean isDisallowGrossSelection() {
		return disallowGrossSelection;
	}

	public boolean isAllowMktSelection() {
		return allowMktSelection;
	}

	public void setDisallowPeriodicitySelection(boolean disallowPeriodicitySelection) {
		this.disallowPeriodicitySelection = disallowPeriodicitySelection;
	}

	public boolean isDisallowPeriodicitySelection() {
		return disallowPeriodicitySelection;
	}

	public void setAllowMktSelection(boolean allowMktSelection) {
		this.allowMktSelection = allowMktSelection;
	}

	public boolean isAllowSorting() {
		return allowSorting;
	}

	public void setAllowSorting(boolean allowSorting) {
		this.allowSorting = allowSorting;
	}

	public void setHasExcelData(boolean hasExcelData) {
		this.hasExcelData = hasExcelData;
	}

	public boolean isHasExcelData() {
		return hasExcelData;
	}
	
	public boolean isPerformanceReport() {
		if ( "P".equals(reportType))
			return true;
		else
			return false;
	}

	public void setUsrSecurityBean(UserSecurityBean usrSecurityBean) {
		this.usrSecurityBean = usrSecurityBean;
	}

	public UserSecurityBean getUsrSecurityBean() {
		return usrSecurityBean;
	}

	public boolean isDisallowGroupbyBenchmark() {
		return disallowGroupbyBenchmark;
	}

	public void setDisallowGroupbyBenchmark(boolean disallowGroupbyBenchmark) {
		this.disallowGroupbyBenchmark = disallowGroupbyBenchmark;
	}

	public String getDisclosure() {
		if(disclosure == null)
			return "";
		return disclosure;
	}

	public boolean isAllowFundAndPlanTypeSelection() {
		return allowFundAndPlanTypeSelection;
	}

	public void setAllowFundAndPlanTypeSelection(
			boolean allowFundAndPlanTypeSelection) {
		this.allowFundAndPlanTypeSelection = allowFundAndPlanTypeSelection;
	}

	public Boolean getProdCheckedAll() {
		return prodCheckedAll;
	}

	public void setProdCheckedAll(Boolean prodCheckedAll) {
		this.prodCheckedAll = prodCheckedAll;
	}
}
